// This file has been prepared for Doxygen automatic documentation generation.
/*! \file ********************************************************************
*
* Atmel Corporation
*
* \li File:               main.c
* \li Compiler:           IAR EWAAVR 3.20a
* \li Support mail:       avr@atmel.com
*
* \li Supported devices:  All AVRs.
*
* \li Application Note:   AVR318 - Dallas 1-Wire(R) master.
*                         
*
* \li Description:        Example on how to use the 1-Wire(R) interface
*                         master.
*
*                         $Revision: 1.7 $
*                         $Date: Thursday, August 19, 2004 14:27:18 UTC $
****************************************************************************/


#include "main.h"

#include "OWIPolled.h"
#include "OWIHighLevelFunctions.h"
#include "OWIBitFunctions.h"
#include "OWIcrc.h"
#include "jb/uhr.h"


// Defines used only in code example.
// Die Family Codes sind in der Datei main.h definiert!
//
#define DS1820_START_CONVERSION			0x44
#define DS1820_READ_SCRATCHPAD			0xbe
#define DS1820_READ_POWER_SUPPLY		0xb4
#define DS1820_ERROR					-1000   // Return code. Outside temperature range.

#define DS2890_WRITE_CONTROL_REGISTER	0X55
#define DS2890_RELEASE_CODE				0x96
#define DS2890_WRITE_POSITION			0x0f

#define OWI_ROM_SKIP					0xcc	//!< SKIP ROM command code.

// Definitionen fr dieses Programm
#define SEARCH_SUCCESSFUL				0x00
#define SEARCH_CRC_ERROR				0x01

#define FALSE		0
#define TRUE		1



/*! \brief  Data type used to hold information about slave devices.
 *  
 *  The OWI_device data type holds information about what bus each device
 *  is connected to, and its 64 bit identifier.
 */


// Prototypes of functions used in exemples.

unsigned char OWI_ComputeCRC8(unsigned char inData, unsigned char seed);
unsigned char eeprom_read_byte_index (char nr, char Subindex);

int OWIcounter;
char OWIstate;
#define OWICOUNTER_800ms_TIMEOUT	(8 * F_CPU / TIMER_PRESCALE / 10)
#define OWICOUNTER_10ms_TIMEOUT		(F_CPU / TIMER_PRESCALE / 100)

void	SearchBuses				(unsigned char buses);
void	DS1820_ReadTemperature	(unsigned char nr);


/*! \brief  Example application for the polled drivers.
 *
 *  Application for the software only driver.
 *  This application will find all devices (upper bounded by MAX_DEVICES-1) 
 *  on the buses defined by BUSES. It then tries to find a DS1820  
 *  device on a bus, and communicate with it to read temperature (DS1820).
 *  This example is not written in a very optimal way. It is merely intended to show
 *  how the polled 1-Wire(R) driver can be used.
 */
void OWI_init(void)
{
	VA4 = DALLAS[0][0] & OWI_PIN_4;	// Wir bernehmen die Initialisierungsdaten aus dem EEPROM in die
	VA5 = DALLAS[0][0] & OWI_PIN_5;	//  entsprechenden Variablen
	VA6 = DALLAS[0][0] & OWI_PIN_3;
	
	OWI_Reset(BUSES);					// Bus initialisieren
	OWIcounter = TCNT1;					// Zeitreferenz initialisieren
	OWIstate = 0;						// Status = reset;
	
	int i;
	for(i = 1; i < MAX_DEVICES; i++) {				// Datenfeld im SRAM initialisieren
		DALLAS[i][1] = 0xff;							// "Baustein ist kein Thermometer" setzen
		DALLAS[i][0] = eeprom_read_byte_index(i,7);	// Kennung merken
	}
}


void OWI_check(void)
{
	if((OWIcounter-TCNT1) > (F_CPU/1024)) 	// falls das timeout erreicht ist,
	switch (OWIstate) {
		case 0:										// Temperaturmessung starten
	BUSES = DALLAS[0][0];									// Busleitungen aktualisieren
	SearchBuses(BUSES); 									// Do the bus search.   
			OWI_DetectPresence(BUSES);								// Bus-Reset
			OWI_SendByte(OWI_ROM_SKIP, BUSES);						// Send skip ROM command. "An Alle"
			OWI_SendByte(DS1820_START_CONVERSION, BUSES);			// Send start conversion command.
			OWI_STRONG_PULLUP(BUSES);								// strong pullup on.
			
			OWIcounter += (OWICOUNTER_800ms_TIMEOUT);	// nach 0,8 s geht es weiter
			OWIstate = 1;
		break;
		
		default:										// Thermometer auslesen
			DS1820_ReadTemperature(OWIstate);
			if (DALLAS[(int)OWIstate][1] == 0x7f) {			// Wenn der Baustein nicht angeschlossen ist
				DS1820_ReadTemperature(OWIstate);				//  noch einmal wiederholen
				if (DALLAS[(int)OWIstate][1] == 0x7f) 		// Wenn sich der Baustein wieder nicht meldet
					DS1820_ReadTemperature(OWIstate);			//  ein zweites mal wiederholen
			}
			if (++OWIstate == MAX_DEVICES) OWIstate = 0;		// Fr alle Tabelleneintrge wiederholen
			OWIcounter += (OWICOUNTER_10ms_TIMEOUT);			// nach 0,01 s geht es weiter
		break;
	}
}

/*
// *********************************************************************************************

*/


/*! \brief  Perform a 1-Wire search
 *
 *  This function shows how the OWI_SearchRom function can be used to 
 *  discover all slaves on the bus. It will also CRC check the 64 bit
 *  identifiers.
 *
 *  \param  devices Pointer to an array of type OWI_device. The discovered 
 *                  devices will be placed from the beginning of this array.
 *
 *  \param  len     The length of the device array. (Max. number of elements).
 *
 *  \param  buses   Bitmask of the buses to perform search on.
 *
 *  \retval SEARCH_SUCCESSFUL   Search completed successfully.
 *  \retval SEARCH_CRC_ERROR    A CRC error occured. Probably because of noise
 *                              during transmission.
 */
void SearchBuses(unsigned char buses)
{
    int i;
    unsigned char * currentID;
    unsigned char j, presence, currentBus, lastDeviation, id[8];
	
	presence = OWI_DetectPresence(BUSES);					// Find slave devices on all buses.
	currentID = id;											// currentID zeigt auf das Datenfeld
	for (currentBus=0x01; currentBus; currentBus<<=1){	// Go through all buses with slave devices.
		lastDeviation = 0;									// Startbedingungen fr den Bus
		if ((currentBus&BUSES) & presence){ 			// If (Bus included in BUSES) and Devices available on any bus.
            do {  // Do slave search on each bus, and place identifiers and corresponding bus "addresses" in the EEPROM.
				OWI_DetectPresence(currentBus);				// Reset auf currentBus auslsen
				lastDeviation = OWI_SearchRom(currentID, lastDeviation, currentBus);	// nchsten Baustein suchen
				Check_Clock();	// mu gelegentlich sein (nach jedem gefundenen Baustein)
				// Jetzt machen wir den CRC-Check und falls wir am Ende der Daten im EEPROM ankommen, 
				// wird (newID) ins EEPROM eintragen. Den EEPROM-Index von newID merken wir uns im RAM.
				if (!OWI_ComputeCRCn(id,8)) {		// Wenn CRC ok, suchen wir currentID im EEPROM
					i = 1;							// Bausteinindex (geht bei 1 los, 0 bleibt unbenutzt!)
					j = 0;							// Index im ID-Feld
					do {							// Fr alle Pltze im EEPROM wiederholen
						i <<= 4;					// Bausteinindex in EEPROM-Adresse umrechnen
						do {						// Fr alle Bytes der ID wiederholen
							if (id[j]==eeprom_read_byte((char *)i+j))	// wenn die Bytes gleich sind,
								j++;										// nchstes Byte vergleichen
							else j=99;				// andernfalls, Vergleich dieser Seriennummer abbrechen
						} while (j<7);			// Es sind 7 Byte zu vergleichen (0..6)
							
						if (j==7) {				// Die Seriennummer wurde gefunden, fertig
							if (eeprom_read_byte ((char *)i+7)!=currentBus)	// Wenn sich der Bus gendert hat
								eeprom_write_byte ((uint8_t *)i+7,currentBus);	// Bitmaske erneuern
						}
						else { 					// wenn j<>7, wurde die Seriennummer nicht gefunden
							if (eeprom_read_byte ((char *)i+7)==Leerkennung) {
								eeprom_write_block(currentID,(char *)i,7);	//  die ID ins EEPROM schreiben
								eeprom_write_byte ((uint8_t *)i+7,currentBus);	//  und die Bitmaske der Portleitung anhngen
								DALLAS[i>>= 4][0] = !Leerkennung; // Platz ist belegt
								j = 7;				// Die Seriennummer steht nun im EEPROM
								//i >>= 4;			// Adresse zurck auf den Bausteinindex setzen (geschenkt)
							}
							else {					// Platz in Gebrauch, weiter nach freiem Platz suchen
								i >>= 4;			// dazu Adresse zurck auf den Bausteinindex setzen
								i += 1;				// Bausteinindex erhhen
								if (i<MAX_DEVICES)	// wenn noch Platz im EEPROM ist, weiteren Suchdurchlauf vorbereiten
									j = 0;			// Index auf den Anfang des ID-Feldes setzen
							}
						}
					} while (j==0);// weitersuchen bis j<>0, d.h. Seriennummer gefunden oder Speicher erschpft
				} // Ende der Suche und des Speicherns im EEPROM
			}	while(lastDeviation != OWI_ROM_SEARCH_FINISHED);
			// Ende: Do slave search on each bus, and place identifiers and corresponding bus "addresses" in the EEPROM.
		}// Ende: If (Bus included in BUSES) and Devices available on any bus.
	}// Ende: Go through all buses with slave devices.
}


// Die Eingangsgren HOB und LOB werden in eine vorzeichenbefaftete Integerzahl gewandelt,
// die stetig mit der Temperatur steigt.
//
int wandeln(char HOB, char LOB)
{
	if (HOB&0x80)											// Bei negativer Temperatur
	{
		HOB = ~HOB;										// Einerkomplement
		HOB |= 0x80;										// Vorzeichen erhalten
		LOB = 100 - (LOB&0x7f);							// Nachkommaanteil komplementieren, Schaltzustand ausblenden
	}
	return ((HOB<<8) | (LOB&0x7f));						// Temperatur zu einem signed int zusammensetzen
}

/*! \brief  Read the temperature from a temperature sensor.
 *
 *  This function will read back the temperature
 *  from a DS1820, DS18S20, DS18B20 or DS1822 temperature sensor.
 *
 *  \param  nr  Index where the temperature sensor is located.
 *  
 *  \return The temperature read in DALLAS[nr,0] and DALLAS[nr,1].
 */
void DS1820_ReadTemperature(unsigned char nr)
{
	unsigned char Feld[9], bus, extpwr, Scratch[9];
	char remainder, temperature;

	char Kennung = DALLAS[nr][0]; 						// Satzinhalt feststellen
	
	if (Kennung<Kennungsgrenze){							// Wenn es sich um einen Dallasbaustein handelt
		eeprom_read_block(Feld,(char*)(int)(nr<<4),8);	// Seriennummer und Bus-Maske lesen
		if (	Feld[0]==DS1820_FAMILY_ID 					// Familycodes prfen und
			||	Feld[0]==DS1822_FAMILY_ID 					// gfs. Daten aus DS18(s|B)20 lesen
			||	Feld[0]==DS18B20_FAMILY_ID){	
			bus = Feld[7] & BUSES;							// Bus-Maske merken und auf geschaltete Ltg. begrenzen
			Feld[7] = OWI_ComputeCRCn(Feld,7);				// CRC ergnzen
			OWI_DetectPresence(bus);						// Reset.
			OWI_MatchRom(Feld, bus);    					// Match id.
			OWI_SendByte(DS1820_READ_POWER_SUPPLY, bus);	// Send read Power supply command.
			extpwr = OWI_ReadBit(bus);						// Read "0"=parasite power; "1"=external power
			OWI_DetectPresence(bus);						// Reset.
			OWI_MatchRom(Feld, bus);    					// Match id.
			OWI_ReadSCRATCHPAD(Scratch, bus);				// Daten auslesen
			if(!OWI_ComputeCRCn(Scratch,9)&&bus) {		// Wenn crc-Summe der Daten stimmt, Temperatur errechnen
				
				//
				//	Bearbeitung des DS1820 / DS18S20
				//
				if (Feld[0] == DS1820_FAMILY_ID) {		// DS1820 ?
					temperature  = Scratch[0] >> 1; 		// 0,5-Stelle entfernen
					temperature |= (Scratch[1] & 0x80);	// Vorzeichen hinzufgen;
					char c = Scratch[7]-Scratch[6];		// Messung
					char a = Scratch[7];					// Stellenwert
					remainder = 0;							// Nachkommaanteil im Binrformat ermitteln
					char i=7;								// Bitzhler fr 6 Bit Nachkommaanteil
					do {
						remainder += remainder;				// Ergebnis um ein Bit schieben
						if (c >= a) {						// Wenn Rest >= Stellenwert, ist die Stelle = 1
							c -= a;							// Rest um den Stellenwert verringern
							if (a) remainder++;			// Wenn der Stellenwert >0 ist, Bit hinzufgen
						}
						a = a/2;							// Stellenwert fr nchste Stelle halbieren
					}	while (i--);						// Fr alle Bits wiederholen
					
					if (remainder<16) {					// Dallasspezifische Stellenwertverschiebung durchfhren
						temperature--;						// Temperatur gfs. korrigieren
						remainder += 64;
					}
					remainder -= 16;						// Nachkommaanteil verschieben
				}	// Ende DS1820/DS18S20
				
				//
				//	Bearbeitung des DS18B20 / DS1822 
				//
				else { 									// DS1822 oder DS18B20
					temperature  = Scratch[0] >> 4; 		// Nachkommaanteil entfernen
					temperature |= Scratch[1] << 4;		// Ganzzahligen Anteil ermitteln
					remainder = (Scratch[0]&0xf)<<2;		// 6 Bit langen binren Nachkommaanteil erzeugen
				}
				
				//
				//	Vorzeichenkorrektur
				//
				if (temperature > 127) {					// Falls die Temperatur negativ ist
					temperature = ~temperature;			// Einerkomplement bilden
					if (remainder == 0) temperature++;	// in Zweierkomplement umwandeln
					else remainder = 64 - remainder ;		// oder nur den Nachkommaanteil abziehen
					temperature |= 0x80;					// Vorzeichen wieder hinzufgen
				}
				
				// Nachkommaanteil * 100 und von binr in dezimal wandeln
				remainder = maleinskommafuenfsechszweifuenf(remainder);
				
				DALLAS[nr][0] = temperature;				// Temperatur in C mit Vorzeichen
				DALLAS[nr][1] = remainder;					// Nachkommaanteil in 0,01C
				if (extpwr!=0) DALLAS[nr][1] |= EIN;		// "External Power Bit" setzen
			}
			else {											// crc-Fehler im Datenfeld
				DALLAS[nr][1] = 0x7f;						// Thermometer ist nicht aktiv
			}
		}
		else	DALLAS[nr][1] = 0xff;						// Baustein ist kein Thermometer und Ende
	} // Ende Kennungen unterhalb der Kennungsgrenze
	
	else {
		
		char Schalter = eeprom_read_byte_index(nr,6);		// Ist ein Schaltelement angegeben?
		char Schaltstellung = DALLAS[(int)Schalter][1];	// Welches Signal liefert das Schaltelement ?
		
		if (Kennung==Portkennung || Kennung==Steckdosenkennung) {	// Ausgabeobjekte bearbeiten
			if (Schalter) {								// Wenn kein manueller Betrieb
				if (Schaltstellung == 0xff) 				// Wenn Schaltstellung unbekannt
					schalte_IO(nr,AUS);					// vorsichtshalber abschalten
				else schalte_IO(nr,Schaltstellung & EIN);	// sonst Schaltstellung des Schaltelementes weitergeben
			}
			else											// wenn manueller Betrieb
			if (Kennung==Portkennung)						// Portstellung auslesen und merken
				DALLAS[nr][1] = read_port(eeprom_read_byte_index(nr,0));
		}
		
		if (Kennung==Schaltuhrkennung) {					// Bearbeitung des Steuerobjektes "Schaltuhr"
			if (Schalter && !(Schaltstellung & EIN))		// Wenn die Schaltuhr ausgeschaltet ist,
				schalte_IO(nr,AUS);						// dann Schaltuhrausgang ausschalten
			else					// Wenn die Zeit = Einschaltzeit
			if ((eeprom_read_byte_index((int)nr,0) == hh)
				&& (eeprom_read_byte_index((int)nr,1) == mm))
				schalte_IO(nr,EIN);						// dann Schaltuhrausgang einschalten
			else					// Wenn die Zeit = Ausschaltzeit
			if ((eeprom_read_byte_index((int)nr,2) == hh)
				&& (eeprom_read_byte_index((int)nr,3) == mm))
				schalte_IO(nr,AUS);						// dann Schaltuhrausgang ausschalten
			else
			if (DALLAS[nr][1] == 0xff) schalte_IO(nr,AUS);
		}
		
		if (Kennung==Thermostatkennung) {					// Bearbeitung des Steuerobjektes "Thermostat"
			char Thi = eeprom_read_byte_index(nr,0);		// Thermometerindex Thi bestimmen
			if (Schalter && !(Schaltstellung & EIN))		// Wenn der Thermostat ausgeschaltet ist,
				schalte_IO(nr,AUS);						// dann Thermostatausgang ausschalten
			else					// Wenn die Temperatur des zugehrigen Thermometers < Einschalttemperatur
			if (wandeln(DALLAS[(int)Thi][0], DALLAS[(int)Thi][1])	// Thermometer
				< wandeln(eeprom_read_byte_index((int)nr,1), eeprom_read_byte_index((int)nr,2))) // T ein
				schalte_IO(nr,EIN);						// dann Thermostatausgang einschalten
			else					// Wenn die Temperatur des zugehrigen Thermometers > Ausschalttemperatur
			if (wandeln(DALLAS[(int)Thi][0], DALLAS[(int)Thi][1])	// Thermometer
				> wandeln(eeprom_read_byte_index((int)nr,3), eeprom_read_byte_index((int)nr,4))) // T aus
				schalte_IO(nr,AUS);						// dann Thermostatausgang ausschalten
		}
		
	}
}	
